# coding=utf-8
from otlmow_model.Classes.Abstracten.AfwijkendeKantopsluiting import AfwijkendeKantopsluiting


# Generated with OTLClassCreator. To modify: extend, do not edit
class TrottoirbandWatergreppelAfw(AfwijkendeKantopsluiting):
    """Afwijkende kantopsluiting, die een trottoirband en een watergreppel combineert in een geheel."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#TrottoirbandWatergreppelAfw'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    def __init__(self):
        super().__init__()

        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Fundering')
        self.add_valid_relation(relation='https://wegenenverkeer.data.vlaanderen.be/ns/onderdeel#Bevestiging', target='https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#KabelgeleidingEnLeidingBevestiging')



import inspect
import warnings
from typing import Type

from otlmow_model.BaseClasses.RelationInteractor import RelationInteractor
from otlmow_model.Classes.ImplementatieElement.RelatieObject import RelatieObject
from otlmow_model.Exceptions.RelationDeprecationWarning import RelationDeprecationWarning


class RelationValidator:
    @staticmethod
    def is_valid_relation_instance(source: RelationInteractor, relation_instance: RelatieObject, target: RelationInteractor):
        return RelationValidator.is_valid_relation(source=source,
                                                   relation=type(relation_instance),
                                                   target=target)

    @staticmethod
    def is_valid_relation(source: RelationInteractor, relation: Type[RelatieObject], target: RelationInteractor):
        if 'lgc.' in source.typeURI or 'lgc.' in target.typeURI:
            return True

        if relation.typeURI not in source._valid_relations:
            return False
        targets = source._valid_relations[relation.typeURI].keys()
        if target.typeURI in targets:
            deprecated_value = source._valid_relations[relation.typeURI][target.typeURI]
            if deprecated_value != '':
                warnings.warn(
                    message=f'the relation of type {relation.typeURI} between assets of types {source.typeURI} and {target.typeURI} is deprecated since version {deprecated_value}',
                    category=RelationDeprecationWarning)
            return True

        bases = inspect.getmro(type(target))
        for base in bases:
            base_type_uri = RelationValidator._get_member(base, 'typeURI')
            if base_type_uri in targets:
                deprecated_value = source._valid_relations[relation.typeURI][base_type_uri]
                if deprecated_value != '':
                    warnings.warn(
                        message=f'the relation of type {relation.typeURI} between assets of types {source.typeURI} and {target.typeURI} is deprecated since version {deprecated_value}',
                        category=RelationDeprecationWarning)
                return True

        # print(bases)
        return False

    @staticmethod
    def _get_member(obj, name):
        return next(iter([member for _name, member in inspect.getmembers(obj) if name == _name]), None)



# coding=utf-8
from abc import abstractmethod
from otlmow_model.Classes.ImplementatieElement.AIMNaamObject import AIMNaamObject
from otlmow_model.GeometrieTypes.PuntGeometrie import PuntGeometrie


# Generated with OTLClassCreator. To modify: extend, do not edit
class Communicatieapparatuur(AIMNaamObject, PuntGeometrie):
    """Abstracte voor alle benodigde apparatuur voor het ontvangen en verzenden van signalen."""

    typeURI = 'https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#Communicatieapparatuur'
    """De URI van het object volgens https://www.w3.org/2001/XMLSchema#anyURI."""

    @abstractmethod
    def __init__(self):
        AIMNaamObject.__init__(self)
        PuntGeometrie.__init__(self)



from collections import defaultdict
from typing import Iterable

from otlmow_model.BaseClasses.OTLObject import OTLObject


def count_assets_by_type(objects: Iterable[OTLObject]) -> defaultdict:
    d = defaultdict(int)
    for i in objects:
        d[i.typeURI] += 1
    return d


def print_overview_assets(objects: Iterable[OTLObject]) -> None:
    for k, v in count_assets_by_type(objects).items():
        print(f'counting {str(v)} assets of type {k}')


def remove_duplicates_in_iterable_based_on_property(iterable: Iterable[OTLObject], property_name: str) -> []:
    d = {}
    for asset in iterable:
        item = asset
        last = property_name
        while '.' in last:
            first = last.split('.')[0]
            last = last.split('.', 1)[1]
            if hasattr(item, first):
                item = getattr(item, first)
                if isinstance(item, list):
                    raise NotImplementedError("can't use remove_duplicates_in_iterable_based_on_property() when the property value is a list")
        if hasattr(item, last):
            item_prop = getattr(item, last)
            if isinstance(item_prop, list):
                raise NotImplementedError(
                    "can't use remove_duplicates_in_iterable_based_on_property() when the property value is a list")
            item_prop_str = str(item_prop)
            if item_prop_str not in d:
                d[item_prop_str] = asset
    return list(d.values())
